package controleurs;

import java.io.IOException;
import java.io.StringReader;
import java.util.Collections;
import java.util.GregorianCalendar;

import modeles.Evenement;
import modeles.ListeEvenement;
import modeles.SemaineCourante;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.ResourceIterator;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;
import org.xmldb.api.modules.XPathQueryService;

import vues.PanelAgenda;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

/**
 * Controleur associe a la fonction agenda du programme
 * @author erik helsens
 */
public class ControleurAgenda {
	/**
	 * la vue de l'agenda
	 */
	private PanelAgenda vue;
	/**
	 * service pour effectuer les requetes dans eXist
	 */
	private XPathQueryService service;
	/** 
	 * le projet XML dans eXist
	 */
	private Collection collection;
	/**
	 *  la liste d'evenements
	 */
	private ListeEvenement evenements;
	/**
	 * objet utilise pour la manipulation des dates
	 */
	private SemaineCourante semaineCourante;
	/**
	 * controleur general de l'application
	 */
	private Controleur controleur;
	/**
	 * id du contact applique comme filtre des evenements (null si pas de filtre)
	 */
	private String idContactFiltre;
	/**
	 * Evenement en cours d'affichage
	 */
	private Evenement evenementAffiche;


	/**
	 * Constructeur du controleur de l'agenda
	 * @param v
	 * @param s
	 * @param c
	 * @param cl
	 */
	public ControleurAgenda(PanelAgenda v, XPathQueryService s, Collection c, Controleur cl) {
		this.vue = v;
		this.service = s;
		this.collection = c;
		this.semaineCourante = new SemaineCourante();
		this.controleur = cl;
		this.idContactFiltre = null;
		this.evenementAffiche = null;
	}


	/**
	 * Effectue une requete sur tous les evenements de la semaine pour initialiser la liste
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	public void requestCurrentWeekEvenements() throws XMLDBException, SAXException, IOException {
		/* Exemple de requête:
		 * 
		   <agenda>
			{for $e in //evenement
				where ($e/date = '11/05/2009' or $e/date = '12/05/2009' or $e/date = '13/05/2009' or $e/date = '14/05/2009' 
						or $e/date = '15/05/2009' or $e/date = '16/05/2009' or $e/date = '17/05/2009')
				return $e }
			</agenda>
		 *
		 */

		// création de la requête
		String requete = "<agenda>\n" +
		"{for $e in //evenement\n" +
		"where (";
		// on l'effectue sur les dates de la semaine
		String[] dates = this.semaineCourante.getWeekStringDates();
		requete +=  "$e/date = '"+dates[0]+"'";
		for(int i = 1; i<dates.length; i++){
			requete += " or $e/date = '"+dates[i]+"'";
		}
		requete += ")";
		// si un filtre sur un contact est appliqué
		if(this.idContactFiltre != null) {
			requete += "and $e/idcontact = '"+this.idContactFiltre+"'";
		}
		requete += "\n" +
		"return $e }\n" +
		"</agenda>";
		// on effectue la requête
		ResourceSet result = service.query(requete);
		// set de l'itérator
		ResourceIterator it = result.getIterator();		
		String xml = "";
		// parcours du résultat
		while (it.hasMoreResources()){
			xml += it.nextResource().getContent().toString() + "\n";
		}
		// on parse le résultat
		DOMParser parser = new DOMParser();
		parser.parse(new InputSource(new java.io.StringReader(xml))); 
		Document doc = parser.getDocument();
		// on set la liste d'evenements du controleur
		this.evenements = new ListeEvenement(doc);
		// tri de la liste d'événements
		Collections.sort(this.evenements);
		// on set la liste d'événements de la vue
		this.vue.setTabList(this.evenements);
	}

	/**
	 * Retourne le nom et le prenom d'un contact a partir de son id
	 * @param id l'id du contact
	 * @return le nom et le prenom du contact
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public String getContactNameById(String id) throws XMLDBException, SAXException, IOException {
		String res = "", tmpN = "", tmpP = "";
		// on effectue les requêtes
		ResourceSet resultNom = service.query("/repertoire/contact[@id="+id+"]/nom");
		ResourceSet resultPrenom = service.query("/repertoire/contact[@id="+id+"]/prenom");
		// set des itérators
		ResourceIterator itNom = resultNom.getIterator();		
		ResourceIterator itPrenom = resultPrenom.getIterator();		
		// parcours des résultat
		while (itNom.hasMoreResources()){
			tmpN += itNom.nextResource().getContent().toString();
		}
		while (itPrenom.hasMoreResources()){
			tmpP += itPrenom.nextResource().getContent().toString();
		}
		// on parse les résultats
		DOMParser parser = new DOMParser();
		parser.parse(new InputSource(new StringReader(tmpN))); 
		Document doc = parser.getDocument();
		res = doc.getDocumentElement().getTextContent();
		parser.parse(new InputSource(new StringReader(tmpP))); 
		doc = parser.getDocument();
		res += " "+doc.getDocumentElement().getTextContent();
		return res;
	}

	/**
	 * Ajoute un Evenement
	 * @param c l'evenement a ajouter
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void addEvenement(Evenement c) throws XMLDBException, SAXException, IOException{
		// récupération de la resource
		XMLResource resource = (XMLResource)this.collection.getResource("agenda.xml");
		// parse en DOM
		Document doc = (Document) resource.getContentAsDOM();
		// ajout du fils
		doc.getDocumentElement().appendChild(ControleurTools.createEvenementElement(c, doc));
		// mise à jour de la ressource
		resource.setContentAsDOM(doc);
		// enregistrement la resource
		this.collection.storeResource(resource);
		// mise à jour de la liste de contacts
		this.requestCurrentWeekEvenements();
	}


	/**
	 * Edite un evenement
	 * @param c l'evenement a editer
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void editEvenement(Evenement c) throws XMLDBException, SAXException, IOException{
		// récupération de la ressource
		XMLResource resource = (XMLResource)this.collection.getResource("agenda.xml");
		// parse en DOM
		Document doc = (Document) resource.getContentAsDOM();
		// noeud à enlever (pour être modifié)
		Node toRemove = ControleurTools.getEvenement(doc, c.getId());
		// remplacement du noeud 
		doc.getDocumentElement().replaceChild(ControleurTools.createEvenementElement(c, doc),toRemove);
		// mise à jour de la resource
		resource.setContentAsDOM(doc);
		// enregistrement de la resource
		this.collection.storeResource(resource);		
		// MAJ de l'affichage
		this.requestCurrentWeekEvenements();
	}

	/**
	 * Enleve un evenement
	 * @param id l'id de l'evenement a enlever
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void removeEvenement(String id) throws XMLDBException, SAXException, IOException{		
		// récupération de la resource
		XMLResource resource = (XMLResource)this.collection.getResource("agenda.xml");
		// parsing en DOM
		Document doc = (Document) resource.getContentAsDOM();
		// noeud à enlever
		Node toRemove = ControleurTools.getEvenement(doc, id);
		// supression du noeud
		doc.getDocumentElement().removeChild(toRemove);
		// mise à jour de la resource
		resource.setContentAsDOM(doc);
		// enregistrement de la resource
		this.collection.storeResource(resource);

		// on supprime l'affichage de l'événement supprimé
		this.vue.clearTextEvenement();
		// mise à jour de la liste des événements
		this.requestCurrentWeekEvenements();
	}

	/**
	 * Passe a la semaine suivante
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void weekAfter() throws XMLDBException, SAXException, IOException {
		// changement de la date de la semaine courante
		this.semaineCourante.weekAfter();
		// MAJ de la liste d'événements
		this.requestCurrentWeekEvenements();
		// MAJ de la vue
		this.vue.changeWeek(this.evenements);
	}

	/**
	 * Passe a la semaine precedente
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void previousWeek() throws XMLDBException, SAXException, IOException {
		// changement de la date de la semaine courante
		this.semaineCourante.previousWeek();
		// MAJ de la liste d'événements
		this.requestCurrentWeekEvenements();
		// MAJ de la vue
		this.vue.changeWeek(this.evenements);
	}
	
	/**
	 * Passe au mois suivant
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void monthAfter() throws XMLDBException, SAXException, IOException {
		// changement de la date de la semaine courante
		this.semaineCourante.monthAfter();
		// MAJ de la liste d'événements
		this.requestCurrentWeekEvenements();
		// MAJ de la vue
		this.vue.changeWeek(this.evenements);
	}

	/**
	 * Passe au mois precedent
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void previousMonth() throws XMLDBException, SAXException, IOException {
		// changement de la date de la semaine courante
		this.semaineCourante.previousMonth();
		// MAJ de la liste d'événements
		this.requestCurrentWeekEvenements();
		// MAJ de la vue
		this.vue.changeWeek(this.evenements);
	}


	/******* 		GETTERS AND SETTERS 	*******/

	public GregorianCalendar getDateLundiSemaineCourante() {
		return semaineCourante.getDateLundi();
	}
	
	public GregorianCalendar getDateDuJour() {
		return semaineCourante.getDateJour();
	}
	
	public int compareDateJour(GregorianCalendar d2){
		return semaineCourante.compareDateJour(d2);
	}

	public GregorianCalendar getCurrentDateByCalendarDayValue(int CalendarDayValue) {
		return semaineCourante.getCurrentDateByCalendarDayValue(CalendarDayValue);
	}

	public Controleur getControleur() {
		return controleur;
	}

	public ListeEvenement getEvenements() {
		return evenements;
	}

	public String getIdContactFiltre() {
		return idContactFiltre;
	}

	public void setIdContactFiltre(String idContactFiltre) {
		this.idContactFiltre = idContactFiltre;
	}


	public Evenement getEvenementAffiche() {
		return evenementAffiche;
	}


	public void setEvenementAffiche(Evenement evenementAffiche) {
		this.evenementAffiche = evenementAffiche;
	}
}
