package controleurs;

import java.io.IOException;
import java.util.Collections;

import modeles.ContactFull;
import modeles.ListeContact;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.ResourceIterator;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;
import org.xmldb.api.modules.XPathQueryService;

import vues.PanelRepertoire;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

/**
 * Controleur associe a la fonction repertoire du programme
 * @author erik helsens
 */
public class ControleurRepertoire {	
	/**
	 *  la vue du repertoire
	 */
	private PanelRepertoire vue;
	/** 
	 * service pour effectuer les requetes dans eXist
	 */
	protected XPathQueryService service;
	/**
	 *  le projet XML dans eXist
	 */
	private Collection collection;
	/**
	 *  la liste de contacts
	 */
	private ListeContact contacts;
	/**
	 * Contact en cours d'affichage
	 */
	private ContactFull contactAffiche;

	/**
	 * Contructeur du controleur du repertoire
	 * @param v la vue du repertoire
	 * @param s le service effectuant les requetes dans eXist
	 * @param c le projet XML dans eXist
	 */
	public ControleurRepertoire(PanelRepertoire v, XPathQueryService s, Collection c) {
		this.vue = v;
		this.service = s;
		this.collection = c;
		this.contactAffiche = null;
	}


	/**
	 * Effectue une requete sur tous les contacts pour initialiser la liste
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	public void requestAllContacts() throws XMLDBException, SAXException, IOException {
		// on effectue la requête
		ResourceSet result = service.query("/repertoire");
		// set de l'itérator
		ResourceIterator it = result.getIterator();		
		String xml = "";
		// parcours du résultat
		while (it.hasMoreResources()){
			xml += it.nextResource().getContent().toString() + "\n";
		}
		// on parse le résultat
		DOMParser parser = new DOMParser();
		parser.parse(new InputSource(new java.io.StringReader(xml))); 
		Document doc = parser.getDocument();
		// on set la liste de contacts du controleur
		this.contacts = new ListeContact(doc);
		// tri de la liste de contacts
		Collections.sort(this.contacts);
		// on set la liste de contacts de la vue
		this.vue.setListe(this.contacts);
	}

	/**
	 * Effectue une requete sur un contact a partir de son id
	 * @param id l'identifiant du contact recherche
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void requestOneContact(String id) throws XMLDBException, SAXException, IOException {
		// requete
		ResourceSet result = service.query("/repertoire/contact[@id="+id+"]");
		// iterator
		ResourceIterator it = result.getIterator();
		String xml = "";
		while (it.hasMoreResources()){
			xml += it.nextResource().getContent().toString() + "\n";
		}
		// on parse le résultat
		DOMParser parser = new DOMParser();
		parser.parse(new InputSource(new java.io.StringReader(xml))); 
		Document doc = parser.getDocument();
		
		this.contactAffiche = new ContactFull(doc.getDocumentElement());
		this.vue.setTextContacts(this.contactAffiche.toString());
	}
	
	/**
	 * Effectue une requete sur tous les contacts pour initialiser la liste
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	public void searchContacts(String nom, String prenom) throws XMLDBException, SAXException, IOException {
		/* Exemple de requête:
		 * 
		   <repertoire>
			{for $e in //contact
				where ($e/nom = 'nom' and $e/prenom = 'prenom')
				return $e }
			</repertoire>
		 *
		 */
		// création de la requête
		String requete = "<repertoire>\n" +
		"{for $e in //contact\n";
		requete += "return if(contains(lower-case($e/nom),lower-case('"+nom+"')) and contains(lower-case($e/prenom),lower-case('"+prenom+"')))";
		requete += "then $e \nelse() }\n" +
					"</repertoire>";
		
		// on effectue la requête
		ResourceSet result = service.query(requete);
		// set de l'itérator
		ResourceIterator it = result.getIterator();		
		String xml = "";
		// parcours du résultat
		while (it.hasMoreResources()){
			xml += it.nextResource().getContent().toString() + "\n";
		}
		// on parse le résultat
		DOMParser parser = new DOMParser();
		parser.parse(new InputSource(new java.io.StringReader(xml))); 
		Document doc = parser.getDocument();
		// on set la liste de contacts du controleur
		this.contacts = new ListeContact(doc);
		// tri de la liste de contacts
		Collections.sort(this.contacts);
		// on set la liste de contacts de la vue
		this.vue.setListe(this.contacts);
	}
	

	/**
	 * Ajoute un contact
	 * @param c le contact a ajouter
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void addContact(ContactFull c) throws XMLDBException, SAXException, IOException{
		// récupération de la resource
		XMLResource resource = (XMLResource)this.collection.getResource("repertoire.xml");
		// parse en DOM
		Document doc = (Document) resource.getContentAsDOM();
		// ajout du fils
		doc.getDocumentElement().appendChild(ControleurTools.createContactElement(c, doc));
		// mise à jour de la ressource
		resource.setContentAsDOM(doc);
		// enregistrement la resource
		this.collection.storeResource(resource);
		// mise à jour de la liste de contacts
		this.requestAllContacts();
	}

	/**
	 * Edite un contact
	 * @param c le contact a editer
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void editContact(ContactFull c) throws XMLDBException, SAXException, IOException{
		// récupération de la ressource
		XMLResource resource = (XMLResource)this.collection.getResource("repertoire.xml");
		// parse en DOM
		Document doc = (Document) resource.getContentAsDOM();
		// noeud à enlever (pour être modifié)
		Node toRemove = ControleurTools.getContact(doc, c.getId());
		// remplacement du noeud 
		doc.getDocumentElement().replaceChild(ControleurTools.createContactElement(c, doc),toRemove);
		// mise à jour de la resource
		resource.setContentAsDOM(doc);
		// enregistrement de la resource
		this.collection.storeResource(resource);
		// MAJ de la liste
		this.requestAllContacts();
	}

	/**
	 * Enleve un contact
	 * @param id l'id du contact a enlever
	 * @throws XMLDBException
	 * @throws SAXException
	 * @throws IOException
	 */
	public boolean removeContact(String id) throws XMLDBException, SAXException, IOException{
		boolean res = false;
		// on vérifie si le contact n'a pas d'événement associé
		ResourceSet result = service.query("//evenement[idcontact='"+id+"']");
		ResourceIterator it = result.getIterator();
		// si le contact n'a pas d'événements associés
		if (!it.hasMoreResources()) {
			res = true;
			// récupération de la resource
			XMLResource resource = (XMLResource)this.collection.getResource("repertoire.xml");
			// parsing en DOM
			Document doc = (Document) resource.getContentAsDOM();
			// noeud à enlever
			Node toRemove = ControleurTools.getContact(doc, id);
			// supression du noeud
			doc.getDocumentElement().removeChild(toRemove);
			// mise à jour de la resource
			resource.setContentAsDOM(doc);
			// enregistrement de la resource
			this.collection.storeResource(resource);

			// on supprime l'affichage du contact supprimé
			this.vue.clearTextContacts();
			// mise à jour de la liste des contacts
			this.requestAllContacts();
		}
		return res;
	}

	/**
	 * Retourne la liste de contacts
	 */
	public ListeContact getContacts() {
		return contacts;
	}


	public ContactFull getContactAffiche() {
		return contactAffiche;
	}


	public void setContactAffiche(ContactFull contactAffiche) {
		this.contactAffiche = contactAffiche;
	}
}
